@@ -29,15 +29,13 @@
 # might involve turning this into a virtual base class, and having
 # UserSet and KeywordSet types that inherit from it.
 
+use diagnostics;
 use strict;
 
-# Everything that uses RelationSet should already have globals.pl loaded
-# so we don't want to load it here.  Doing so causes a loop in Perl because
-# globals.pl turns around and does a 'use RelationSet'
-# See http://bugzilla.mozilla.org/show_bug.cgi?id=72862
-#require "globals.pl";
+require "globals.pl";
 
 package RelationSet;
+use CGI::Carp qw(fatalsToBrowser);
 
 # create a new empty RelationSet
 #
@@ -101,8 +99,8 @@ sub generateSqlDeltas {
   if ( $#insertList > -1 ) {
     $sqlInsert = "INSERT INTO $table ($invariantName, $columnName) VALUES " .
       join (",", 
-            map ( "($invariantValue, $_)" , @insertList ) 
-           );
+	    map ( "($invariantValue, $_)" , @insertList ) 
+	   );
   }
      
   # construct the delete list by seeing which relations exist in the
@@ -246,7 +244,7 @@ sub toArrayOfStrings {
     push @result, &::DBID_to_name($i);
   }
 
-  return sort { lc($a) cmp lc($b) } @result;
+  return sort(@result);
 }  
 
 # return this set in string form (comma-separated and sorted)
@@ -262,5 +260,3 @@ sub toString {
 
   return join(',', sort(@result));
 }  
-
-1;
