@@ -18,6 +18,7 @@
 # Rights Reserved.
 #
 # Contributor(s): Terry Weissman <terry@mozilla.org>
+#                 Dave Miller <justdave@syndicomm.com>
 
 use diagnostics;
 use strict;
@@ -38,7 +39,7 @@ sub bug_form_pl_sillyness {
     $zz = @::legal_platform;
     $zz = @::legal_product;
     $zz = @::legal_priority;
-    $zz = @::legal_resolution_no_dup;
+    $zz = @::settable_resolution;
     $zz = @::legal_severity;
     $zz = %::target_milestone;
 }
@@ -118,6 +119,7 @@ my $assignedtoid = $bug{'assigned_to'};
 my $reporterid = $bug{'reporter'};
 my $qacontactid =  $bug{'qa_contact'};
 
+$bug{'assigned_to_email'} = DBID_to_name($assignedtoid);
 $bug{'assigned_to'} = DBID_to_real_or_loginname($bug{'assigned_to'});
 $bug{'reporter'} = DBID_to_real_or_loginname($bug{'reporter'});
 
@@ -139,8 +141,6 @@ GetVersionTable();
 # These should be read from the database ...
 #
 
-my $resolution_popup = make_options(\@::legal_resolution_no_dup,
-				    $bug{'resolution'});
 my $platform_popup = make_options(\@::legal_platform, $bug{'rep_platform'});
 my $priority_popup = make_options(\@::legal_priority, $bug{'priority'});
 my $sev_popup = make_options(\@::legal_severity, $bug{'bug_severity'});
@@ -151,11 +151,18 @@ my $component_popup = make_options($::components{$bug{'product'}},
 
 my $ccSet = new RelationSet;
 $ccSet->mergeFromDB("select who from cc where bug_id=$id");
-my $cc_element = '<INPUT NAME=cc SIZE=30 VALUE="' .
-  $ccSet->toString() . '">';
+my @ccList = $ccSet->toArrayOfStrings();
+my $cc_element = "<INPUT TYPE=HIDDEN NAME=cc VALUE=\"\">";
+if (scalar(@ccList) > 0) {
+  $cc_element = "<SELECT NAME=cc MULTIPLE SIZE=5>\n";
+  foreach my $ccName ( @ccList ) {
+    $cc_element .= "<OPTION VALUE=\"$ccName\">$ccName\n";
+  }
+  $cc_element .= "</SELECT><BR>\n" .
+        "<INPUT TYPE=CHECKBOX NAME=removecc>Remove selected CCs<br>\n";
+}
 
-
-my $URL = $bug{'bug_file_loc'};
+my $URL = value_quote($bug{'bug_file_loc'});
 
 if (defined $URL && $URL ne "none" && $URL ne "NULL" && $URL ne "") {
     $URL = "<B><A HREF=\"$URL\">URL:</A></B>";
@@ -168,41 +175,56 @@ print "
 <INPUT TYPE=HIDDEN NAME=\"longdesclength\" VALUE=\"$longdesclength\">
 <INPUT TYPE=HIDDEN NAME=\"id\" VALUE=$id>
   <TABLE CELLSPACING=0 CELLPADDING=0 BORDER=0><TR>
-    <TD ALIGN=RIGHT><B>Bug#:</B></TD><TD><A HREF=\"show_bug.cgi?id=$bug{'bug_id'}\">$bug{'bug_id'}</A></TD>
+    <TD ALIGN=RIGHT><B>Bug#:</B></TD><TD><A HREF=\"" . Param('urlbase') . "show_bug.cgi?id=$bug{'bug_id'}\">$bug{'bug_id'}</A></TD>
+  <TD>&nbsp;</TD>
     <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html#rep_platform\">Platform:</A></B></TD>
     <TD><SELECT NAME=rep_platform>$platform_popup</SELECT></TD>
-    <TD ALIGN=RIGHT><B>Version:</B></TD>
-    <TD><SELECT NAME=version>" .
-    make_options($::versions{$bug{'product'}}, $bug{'version'}) .
-    "</SELECT></TD>
-  </TR><TR>
+  <TD>&nbsp;</TD>
+    <TD ALIGN=RIGHT><B>Reporter:</B></TD><TD>$bug{'reporter'}</TD>
+</TR><TR>
     <TD ALIGN=RIGHT><B>Product:</B></TD>
     <TD><SELECT NAME=product>" .
     make_options(\@::legal_product, $bug{'product'}) .
     "</SELECT></TD>
+  <TD>&nbsp;</TD>
     <TD ALIGN=RIGHT><B>OS:</B></TD>
     <TD><SELECT NAME=op_sys>" .
     make_options(\@::legal_opsys, $bug{'op_sys'}) .
-    "</SELECT><TD ALIGN=RIGHT><B>Reporter:</B></TD><TD>$bug{'reporter'}</TD>
-     </TDTR><TR>
-    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html\">Status:</A></B></TD>
-      <TD>$bug{'bug_status'}</TD>
-    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html#priority\">Priority:</A></B></TD>
-      <TD><SELECT NAME=priority>$priority_popup</SELECT></TD>
-    <TD ALIGN=RIGHT><B>Cc:</B></TD>
-      <TD> $cc_element </TD>
-  </TR><TR>
-    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html\">Resolution:</A></B></TD>
-      <TD>$bug{'resolution'}</TD>
-    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html#severity\">Severity:</A></B></TD>
-      <TD><SELECT NAME=bug_severity>$sev_popup</SELECT></TD>
+    "</SELECT></TD>
+  <TD>&nbsp;</TD>
+    <TD ALIGN=RIGHT NOWRAP><b>Add CC:</b></TD>
+    <TD><INPUT NAME=newcc SIZE=30 VALUE=\"\"></TD>
+</TR><TR>
     <TD ALIGN=RIGHT><B><A HREF=\"describecomponents.cgi?product=" .
     url_quote($bug{'product'}) . "\">Component:</A></B></TD>
       <TD><SELECT NAME=component>$component_popup</SELECT></TD>
-  </TR><TR>
+  <TD>&nbsp;</TD>
+    <TD ALIGN=RIGHT><B>Version:</B></TD>
+    <TD><SELECT NAME=version>" .
+    make_options($::versions{$bug{'product'}}, $bug{'version'}) .
+    "</SELECT></TD>
+  <TD>&nbsp;</TD>
+    <TD ROWSPAN=4 ALIGN=RIGHT VALIGN=TOP><B>Cc:</B></TD>
+    <TD ROWSPAN=4 VALIGN=TOP> $cc_element </TD>
+</TR><TR>
+    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html\">Status:</A></B></TD>
+      <TD>$bug{'bug_status'}</TD>
+  <TD>&nbsp;</TD>
+    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html#priority\">Priority:</A></B></TD>
+      <TD><SELECT NAME=priority>$priority_popup</SELECT></TD>
+  <TD>&nbsp;</TD>
+</TR><TR>
+    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html\">Resolution:</A></B></TD>
+      <TD>$bug{'resolution'}</TD>
+  <TD>&nbsp;</TD>
+    <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html#severity\">Severity:</A></B></TD>
+      <TD><SELECT NAME=bug_severity>$sev_popup</SELECT></TD>
+  <TD>&nbsp;</TD>
+</TR><TR>
     <TD ALIGN=RIGHT><B><A HREF=\"bug_status.html#assigned_to\">Assigned&nbsp;To:
         </A></B></TD>
-      <TD>$bug{'assigned_to'}</TD>";
+      <TD>$bug{'assigned_to'}</TD>
+  <TD>&nbsp;</TD>";
 
 if (Param("usetargetmilestone")) {
     my $url = "";
@@ -220,8 +242,9 @@ if (Param("usetargetmilestone")) {
 <TD><SELECT NAME=target_milestone>" .
     make_options($::target_milestone{$bug{'product'}},
                  $bug{'target_milestone'}) .
-                     "</SELECT></TD>";
-}
+                     "</SELECT></TD>
+  <TD>&nbsp;</TD>";
+} else { print "<TD></TD><TD></TD><TD>&nbsp;</TD>"; }
 
 print "
 </TR>";
@@ -231,10 +254,10 @@ if (Param("useqacontact")) {
     print "
   <TR>
     <TD ALIGN=\"RIGHT\"><B>QA Contact:</B>
-    <TD COLSPAN=6>
+    <TD COLSPAN=7>
       <INPUT NAME=qa_contact VALUE=\"" .
     value_quote($name) .
-    "\" SIZE=60></
+    "\" SIZE=60></TD>
   </TR>";
 }
 
@@ -242,11 +265,11 @@ if (Param("useqacontact")) {
 print "
   <TR>
     <TD ALIGN=\"RIGHT\">$URL
-    <TD COLSPAN=6>
-      <INPUT NAME=bug_file_loc VALUE=\"$bug{'bug_file_loc'}\" SIZE=60></TD>
+    <TD COLSPAN=7>
+      <INPUT NAME=bug_file_loc VALUE=\"" . value_quote($bug{'bug_file_loc'}) . "\" SIZE=60></TD>
   </TR><TR>
     <TD ALIGN=\"RIGHT\"><B>Summary:</B>
-    <TD COLSPAN=6>
+    <TD COLSPAN=7>
       <INPUT NAME=short_desc VALUE=\"" .
     value_quote($bug{'short_desc'}) .
     "\" SIZE=60></TD>
@@ -256,10 +279,10 @@ if (Param("usestatuswhiteboard")) {
     print "
   <TR>
     <TD ALIGN=\"RIGHT\"><B>Status Whiteboard:</B>
-    <TD COLSPAN=6>
+    <TD COLSPAN=7>
       <INPUT NAME=status_whiteboard VALUE=\"" .
     value_quote($bug{'status_whiteboard'}) .
-    "\" SIZE=60></
+    "\" SIZE=60></TD>
   </TR>";
 }
 
@@ -275,47 +298,36 @@ if (@::legal_keywords) {
     my $value = value_quote(join(', ', @list));
     print qq{
 <TR>
-<TD ALIGN=right><B><A HREF="describekeywords.cgi">Keywords</A>:</B>
-<TD COLSPAN=6><INPUT NAME="keywords" VALUE="$value" SIZE=60></TD>
+<TD ALIGN=right><B><A HREF="describekeywords.cgi">Keywords:</A></B>
+<TD COLSPAN=7><INPUT NAME="keywords" VALUE="$value" SIZE=60></TD>
 </TR>
 };
 }
 
 print "<tr><td align=right><B>Attachments:</b></td>\n";
-SendSQL("select attach_id, creation_ts, description from attachments where bug_id = $id");
+SendSQL("select attach_id, creation_ts, mimetype, description from attachments where bug_id = $id");
 while (MoreSQLData()) {
-    my ($attachid, $date, $desc) = (FetchSQLData());
+    my ($attachid, $date, $mimetype, $desc) = (FetchSQLData());
     if ($date =~ /^(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/) {
         $date = "$3/$4/$2 $5:$6";
     }
     my $link = "showattachment.cgi?attach_id=$attachid";
     $desc = value_quote($desc);
-    print qq{<td><a href="$link">$date</a></td><td colspan=4>$desc</td></tr><tr><td></td>};
+    print qq{<td><a href="$link">$date</a></td><td colspan=6>$desc&nbsp;&nbsp;&nbsp;($mimetype)</td></tr><tr><td></td>};
 }
-print "<td colspan=6><a href=\"createattachment.cgi?id=$id\">Create a new attachment</a> (proposed patch, testcase, etc.)</td></tr></table>\n";
+print "<td colspan=7><a href=\"createattachment.cgi?id=$id\">Create a new attachment</a> (proposed patch, testcase, etc.)</td></tr></table>\n";
 
 
 sub EmitDependList {
     my ($desc, $myfield, $targetfield) = (@_);
     print "<th align=right>$desc:</th><td>";
     my @list;
-    SendSQL("select dependencies.$targetfield, bugs.bug_status
- from dependencies, bugs
- where dependencies.$myfield = $id
-   and bugs.bug_id = dependencies.$targetfield
- order by dependencies.$targetfield");
+    SendSQL("select $targetfield from dependencies where  
+             $myfield = $id order by $targetfield");
     while (MoreSQLData()) {
-        my ($i, $stat) = (FetchSQLData());
+        my ($i) = (FetchSQLData());
         push(@list, $i);
-        my $opened = ($stat eq "NEW" || $stat eq "ASSIGNED" ||
-                      $stat eq "REOPENED");
-        if (!$opened) {
-            print "<strike>";
-        }
-        print qq{<a href="show_bug.cgi?id=$i">$i</a>};
-        if (!$opened) {
-            print "</strike>";
-        }
+        print GetBugLink($i, $i);
         print " ";
     }
     print "</td><td><input name=$targetfield value=\"" .
@@ -352,7 +364,7 @@ print "
 <br>
 <B>Additional Comments:</B>
 <BR>
-<TEXTAREA WRAP=HARD NAME=comment ROWS=5 COLS=80></TEXTAREA><BR>";
+<TEXTAREA WRAP=HARD NAME=comment ROWS=10 COLS=80></TEXTAREA><BR>";
 
 
 if ($::usergroupset ne '0') {
@@ -399,6 +411,9 @@ if ($status eq $::unconfirmedstate) {
     }
 }
 
+my $movers = Param("movers");
+$movers =~ s/\s?,\s?/|/g;
+$movers =~ s/@/\@/g;
 
 if ($canedit || $::userid == $assignedtoid ||
       $::userid == $reporterid || $::userid == $qacontactid) {
@@ -418,6 +433,8 @@ if ($canedit || $::userid == $assignedtoid ||
             print "<b>$bug{'resolution'}</b>)<br>\n";
             $knum++;
         }
+        my $resolution_popup = make_options(\@::settable_resolution,
+                                            $bug{'resolution'});
         print "<INPUT TYPE=radio NAME=knob VALUE=resolve>
         Resolve bug, changing <A HREF=\"bug_status.html\">resolution</A> to
         <SELECT NAME=resolution
@@ -428,11 +445,7 @@ if ($canedit || $::userid == $assignedtoid ||
         Resolve bug, mark it as duplicate of bug # 
         <INPUT NAME=dup_id SIZE=6 ONCHANGE=\"document.changeform.knob\[$knum\].checked=true\"><br>\n";
         $knum++;
-        if ( $bug{'assigned_to'} =~ /(.*)\((.*)\)/ ) {
-           $bug{'assigned_to'} = $1;
-           chop($bug{'assigned_to'});
-        }
-        my $assign_element = "<INPUT NAME=\"assigned_to\" SIZE=32 ONCHANGE=\"document.changeform.knob\[$knum\].checked=true\" VALUE=\"$bug{'assigned_to'}\">";
+        my $assign_element = "<INPUT NAME=\"assigned_to\" SIZE=32 ONCHANGE=\"document.changeform.knob\[$knum\].checked=true\" VALUE=\"$bug{'assigned_to_email'}\">";
 
         print "<INPUT TYPE=radio NAME=knob VALUE=reassign> 
           <A HREF=\"bug_status.html#assigned_to\">Reassign</A> bug to
@@ -443,11 +456,29 @@ if ($canedit || $::userid == $assignedtoid ||
         }
         $knum++;
         print "<INPUT TYPE=radio NAME=knob VALUE=reassignbycomponent>
-          Reassign bug to owner of selected component<br>\n";
+          Reassign bug to owner ";
+        if (Param("useqacontact")) { print "and QA contact "; }
+        print "of selected component<br>\n";
         if ($status eq $::unconfirmedstate && ($canconfirm || $canedit)) {
             print "&nbsp;&nbsp;&nbsp;&nbsp;<INPUT TYPE=checkbox NAME=compconfirm> and confirm bug (change status to <b>NEW</b>)<BR>";
         }
         $knum++;
+    } elsif ( Param("move-enabled") && ($bug{'resolution'} eq "MOVED") ) {
+        if ( (defined $::COOKIE{"Bugzilla_login"}) 
+             && ($::COOKIE{"Bugzilla_login"} =~ /($movers)/) ){
+          print "<INPUT TYPE=radio NAME=knob VALUE=reopen> Reopen bug<br>\n";
+          $knum++;
+          if ($status eq "RESOLVED") {
+              print "<INPUT TYPE=radio NAME=knob VALUE=verify>
+          Mark bug as <b>VERIFIED</b><br>\n";
+              $knum++;
+          }
+          if ($status ne "CLOSED") {
+              print "<INPUT TYPE=radio NAME=knob VALUE=close>
+          Mark bug as <b>CLOSED</b><br>\n";
+              $knum++;
+          }
+        }
     } else {
         print "<INPUT TYPE=radio NAME=knob VALUE=reopen> Reopen bug<br>\n";
         $knum++;
@@ -467,13 +498,24 @@ if ($canedit || $::userid == $assignedtoid ||
 print "
 <INPUT TYPE=\"submit\" VALUE=\"Commit\">
 <INPUT TYPE=\"reset\" VALUE=\"Reset\">
-<INPUT TYPE=hidden name=form_name VALUE=process_bug>
-<BR>
+<INPUT TYPE=\"hidden\" name=\"form_name\" VALUE=\"process_bug\">
+<P>
 <FONT size=\"+1\"><B>
  <A HREF=\"show_activity.cgi?id=$id\">View Bug Activity</A>
+ &nbsp; | &nbsp;
  <A HREF=\"long_list.cgi?buglist=$id\">Format For Printing</A>
-</B></FONT><BR>
-</FORM>
+</B></FONT>
+";
+
+if ( Param("move-enabled") && (defined $::COOKIE{"Bugzilla_login"}) && ($::COOKIE{"Bugzilla_login"} =~ /($movers)/) ){
+  print "&nbsp; <FONT size=\"+1\"><B> | </B></FONT> &nbsp;"
+       ."<INPUT TYPE=\"SUBMIT\" NAME=\"action\" VALUE=\"" 
+       . Param("move-button-text") . "\">\n";
+}
+
+print "<BR></FORM>";
+
+print "
 <table><tr><td align=left><B>Description:</B></td>
 <td align=right width=100%>Opened: $bug{'creation_ts'}</td></tr></table>
 <HR>
