@@ -21,10 +21,17 @@
 #                 Dawn Endico <endico@mozilla.org>
 #                 Dan Mosedale <dmose@mozilla.org>
 #                 Joe Robins <jmrobins@tgix.com>
+#                 Jake <jake@acutex.net>
+#
 
 # This file defines all the parameters that we have a GUI to edit within
 # Bugzilla.
 
+# ATTENTION!!!!   THIS FILE ONLY CONTAINS THE DEFAULTS.
+# You cannot change your live settings by editing this file.
+# Only adding new parameters is done here.  Once the parameter exists, you 
+# must use %baseurl%/editparams.cgi from the web to edit the settings.
+
 use diagnostics;
 use strict;
 
@@ -113,10 +120,6 @@ sub check_shadowdb {
 # t -- A short text entry field (suitable for a single line)
 # l -- A long text field (suitable for many lines)
 # b -- A boolean value (either 1 or 0)
-# i -- An integer.
-# defenum -- This param defines an enum that defines a column in one of
-#	     the database tables.  The name of the parameter is of the form
-#	     "tablename.columnname".
 
 DefParam("maintainer",
 	 "The email address of the person who maintains this installation of Bugzilla.",
@@ -203,11 +206,46 @@ sub check_despotbaseurl {
 }
 
 
+# Adding in four parameters for LDAP authentication.  -JMR, 7/28/00
+DefParam("useLDAP",
+         "Turn this on to use an LDAP directory for user authentication ".
+         "instead of the Bugzilla database.  (User profiles will still be ".
+         "stored in the database, and will match against the LDAP user by ".
+         "email address.)",
+         "b",
+         0);
+
+
+DefParam("LDAPserver",
+         "The name (and optionally port) of your LDAP server. (e.g. ldap.company.com, or ldap.company.com:portnum)",
+         "t",
+         "");
+
+
+DefParam("LDAPBaseDN",
+         "The BaseDN for authenticating users against. (e.g. \"ou=People,o=Company\")",
+         "t",
+         "");
+
+
+DefParam("LDAPmailattribute",
+         "The name of the attribute of a user in your directory that ".
+         "contains the email address.",
+         "t",
+         "mail");
+#End of LDAP parameters
+
+
 DefParam("headerhtml",
          "Additional HTML to add to the HEAD area of documents, eg. links to stylesheets.",
          "l",
          '');
 
+DefParam("bodyhtml",
+         "Additional parameters to add to the BODY tag at the beginning of documents, eg. background image/colors, link colors, etc",
+         "l",
+         'BGCOLOR="#FFFFFF" TEXT="#000000" LINK="#0000EE" VLINK="#551A8B" ALINK="#FF0000"');
+
 DefParam("footerhtml",
          "HTML to add to the bottom of every page. By default it displays the blurbhtml, and %commandmenu%, a menu of useful commands.  You probably really want either headerhtml or footerhtml to include %commandmenu%.",
          "l",
@@ -249,6 +287,55 @@ information about what Bugzilla is and what it can do, see
 <A HREF=\"http://www.mozilla.org/bugs/\"><B>bug pages</B></A>.");
 
 
+DefParam("mostfreqhtml",
+         "The HTML which appears at the top of the list of most-frequently-reported bugs. Use it to explain the page, set a maintainer etc.",
+         "l",
+         q{
+<br><p>
+
+<b>What are "most frequent bugs"?</b>
+
+<blockquote>The Most Frequent Bugs page lists the known open bugs which 
+are reported most frequently in recent builds of Mozilla. It is automatically
+generated from the Bugzilla database every 24 hours, by counting the number
+of direct and indirect duplicates of bugs.
+This information is provided in order to assist in minimizing
+the amount of duplicate bugs entered into Bugzilla which in turn cuts down
+on development time.
+</blockquote>
+
+<b>How do I use this list?</b>
+
+<ul>
+<li>Review the most frequent bugs list.</li>
+<li>If problem is listed:</li>
+
+<ul>
+<li>Click on Bug # link to confirm that you have found the same bug and comment
+if you have additional information. Or move on with your testing
+of the product.</li>
+</ul>
+
+<li>If problem not listed:</li>
+
+<ul>
+<li>Go to the <a href="query.cgi">Bugzilla Query/Search</a>
+page to try and locate a similar bug that has already been written.</li>
+<li>If you find your bug in Bugzilla, feel free to comment with any new or
+additional data you may have.</li>
+<li>If you cannot find your problem already documented in Bugzilla, go to the
+<a href="http://www.mozilla.org/quality/help/bug-form.html">Bugzilla Helper</a> and post a new bug.</li>
+</ul>
+
+</ul>
+<br>
+});
+
+DefParam("mostfreqthreshold",
+         "The minimum number of duplicates a bug needs to show up on the <A HREF=\"duplicates.cgi\">most frequently reported bugs page</a>. If you have a large database and this page takes a long time to load, try increasing this number.",
+         "t",
+         "2");
+
 DefParam("mybugstemplate",
          "This is the URL to use to bring up a simple 'all of my bugs' list for a user.  %userid% will get replaced with the login name of a user.",
          "t",
@@ -261,6 +348,10 @@ DefParam("shutdownhtml",
          "l",
          "");
 
+DefParam("sendmailnow",
+         "If this is on, Bugzilla will tell sendmail to send any e-mail immediately. If you have a large number of users with a large amount of e-mail traffic, enabling this option may dramatically slow down Bugzilla. Best used for smaller installations of Bugzilla.",
+         "b",
+         0);
 
 DefParam("passwordmail",
 q{The email that gets sent to people to tell them their password.  Within
@@ -291,8 +382,8 @@ separated by a comma (with duplication removed, if they're the same
 person).  %cc% gets replaced by the list of people on the CC list,
 separated by commas.  %bugid% gets replaced by the bug number.
 %diffs% gets replaced by the diff text from the old version to the new
-version of this bug.  %neworchanged% is either "New" or "Changed",
-depending on whether this mail is reporting a new bug or changes made
+version of this bug.  %neworchanged% is "New: " if this mail is
+reporting a new bug or empty if changes were made
 to an existing one.  %summary% gets replaced by the summary of this
 bug.  %<i>anythingelse</i>% gets replaced by the definition of that
 parameter (as defined on this page).},
@@ -300,20 +391,27 @@ parameter (as defined on this page).},
 "From: bugzilla-daemon
 To: %to%
 Cc: %cc%
-Subject: [Bug %bugid%] %neworchanged% - %summary%
+Subject: [Bug %bugid%] %neworchanged%%summary%
 
 %urlbase%show_bug.cgi?id=%bugid%
 
 %diffs%");
 
 DefParam("newemailtech",
-q{There is now experimental code in Bugzilla to do the email diffs in a 
-new and exciting way.  But this stuff is not very cooked yet.  So, right
-now, to use it, the maintainer has to turn on this checkbox, and each user
-has to then turn on the "New email tech" preference.},
+q{The way that email diffs are constructed by Bugzilla.  You can revert to
+the old technology by turning this off, but this is not advised, as the old 
+email tech will probably disappear in the next version of Bugzilla.  Some
+features (watches, server side mail filtering) depend on newemailtech being
+set to on.},
     "b",
-    0);
+    1);
 
+DefParam("newemailtechdefault",
+q{Make "newemailtech" the default for all new accounts created.  This will
+not change any existing accounts nor will it remove a users ability to go
+back to the oldmail system (requires "newemailtech" to be on as well).},
+    "b",
+    1);
 
 DefParam("newchangedmail",
 q{The same as 'changedmail', but used for the newemailtech stuff.},
@@ -321,7 +419,7 @@ q{The same as 'changedmail', but used for the newemailtech stuff.},
 "From: bugzilla-daemon
 To: %to%
 Cc: %cc%
-Subject: [Bug %bugid%] %neworchanged% - %summary%
+Subject: [Bug %bugid%] %neworchanged%%summary%
 
 %urlbase%show_bug.cgi?id=%bugid%
 
@@ -387,7 +485,7 @@ DefParam("defaultquery",
 
 
 DefParam("letsubmitterchoosepriority",
-         "If this is on, then people submitting bugs can choose an initial priority for that bug.  If off, then all bugs initially have the default priority selected above.",
+         "If this is on, then people submitting bugs can choose an initial priority for that bug.  If off, then all bugs initially have the default priority selected below.",
          "b",
          1);
 
@@ -427,6 +525,11 @@ DefParam("curmilestone",
          1,
          \&check_numeric);
 
+DefParam("musthavemilestoneonaccept",
+         "If you are using Target Milestone, do you want to require that the milestone be set in order for a user to ACCEPT a bug?",
+         "b",
+         0);
+
 DefParam("useqacontact",
 	 "Do you wish to use the QA Contact field?",
 	 "b",
@@ -555,5 +658,41 @@ DefParam("supportwatchers",
          " about) another's bugs.  Useful for people going on vacation, and" .
          " QA folks watching particular developers' bugs",
          "b", 0 );
-1;
 
+
+DefParam("move-enabled",
+         "If this is on, Bugzilla will allow certain people to move bugs to the defined database.",
+         "b",
+	 0);
+DefParam("move-button-text",
+         "The text written on the Move button. Explain where the bug is being moved to.",
+         "t",
+         'Move To Bugscape');
+DefParam("move-to-url",
+         "The URL of the database we allow some of our bugs to be moved to.",
+         "t",
+         '');
+DefParam("move-to-address",
+         "To move bugs, an email is sent to the target database. This is the email address that database
+          uses to listen for incoming bugs.",
+         "t",
+         'bugzilla-import');
+DefParam("moved-from-address",
+         "To move bugs, an email is sent to the target database. This is the email address from which
+          this mail, and error messages are sent.",
+         "t",
+         'bugzilla-admin');
+DefParam("movers",
+         "A list of people with permission to move bugs and reopen moved bugs (in case the move operation fails).",
+         "t",
+         '');
+DefParam("moved-default-product",
+         "Bugs moved from other databases to here are assigned to this product.",
+         "t",
+         '');
+DefParam("moved-default-component",
+         "Bugs moved from other databases to here are assigned to this component.",
+         "t",
+         '');
+
+1;
