@@ -19,6 +19,7 @@
 #
 # Contributor(s): Terry Weissman <terry@mozilla.org>
 #                 Dan Mosedale <dmose@mozilla.org>
+#                 Jake <jake@acutex.net>
 
 # Contains some global variables and routines used throughout bugzilla.
 
@@ -54,14 +55,14 @@ sub globals_pl_sillyness {
 # here
 # 
 
-my $db_host = "localhost";
-my $db_name = "bugs";
-my $db_user = "bugs";
-my $db_pass = "";
+$::db_host = "localhost";
+$::db_name = "bugs";
+$::db_user = "bugs";
+$::db_pass = "";
 
 do 'localconfig';
 
-use Mysql;
+use DBI;
 
 use Date::Format;               # For time2str().
 use Date::Parse;               # For str2time().
@@ -69,7 +70,7 @@ use Date::Parse;               # For str2time().
 use RelationSet;
 
 # Contains the version string for the current running Bugzilla.
-$::param{'version'} = '2.10';
+$::param{'version'} = '2.12';
 
 $::dontchange = "--do_not_change--";
 $::chooseone = "--Choose_one:--";
@@ -77,16 +78,22 @@ $::defaultqueryname = "(Default query)";
 $::unconfirmedstate = "UNCONFIRMED";
 $::dbwritesallowed = 1;
 
+# Adding a global variable for the value of the superuser groupset.
+# Joe Robins, 7/5/00
+$::superusergroupset = "9223372036854775807";
+
 sub ConnectToDatabase {
     my ($useshadow) = (@_);
     if (!defined $::db) {
-        my $name = $db_name;
+        my $name = $::db_name;
         if ($useshadow && Param("shadowdb") && Param("queryagainstshadowdb")) {
             $name = Param("shadowdb");
             $::dbwritesallowed = 0;
         }
-	$::db = Mysql->Connect($db_host, $name, $db_user, $db_pass)
-            || die "Can't connect to database server.";
+	$::db = DBI->connect("DBI:mysql:host=$::db_host;database=$name", $::db_user, $::db_pass)
+	    || die "Bugzilla is currently broken. Please try again later. " . 
+      "If the problem persists, please contact " . Param("maintainer") .
+      ". The error you should quote is: " . $DBI::errstr;
     }
 }
 
@@ -104,7 +111,31 @@ sub SyncAnyPendingShadowChanges {
         $shadowchanges = 0;
     }
 }
-    
+
+
+# This is used to manipulate global state used by SendSQL(),
+# MoreSQLData() and FetchSQLData().  It provides a way to do another
+# SQL query without losing any as-yet-unfetched data from an existing
+# query.  Just push the current global state, do your new query and fetch
+# any data you need from it, then pop the current global state.
+# 
+@::SQLStateStack = ();
+
+sub PushGlobalSQLState() {
+    push @::SQLStateStack, $::currentquery;
+    push @::SQLStateStack, [ @::fetchahead ]; 
+}
+
+sub PopGlobalSQLState() {
+    die ("PopGlobalSQLState: stack underflow") if ( $#::SQLStateStack < 1 );
+    @::fetchahead = @{pop @::SQLStateStack};
+    $::currentquery = pop @::SQLStateStack;
+}
+
+sub SavedSQLStates() {
+    return ($#::SqlStateStack + 1) / 2;
+}
+
 
 my $dosqllog = (-e "data/sqllog") && (-w "data/sqllog");
 
@@ -113,13 +144,21 @@ sub SqlLog {
         my ($str) = (@_);
         open(SQLLOGFID, ">>data/sqllog") || die "Can't write to data/sqllog";
         if (flock(SQLLOGFID,2)) { # 2 is magic 'exclusive lock' const.
+
+            # if we're a subquery (ie there's pushed global state around)
+            # indent to indicate the level of subquery-hood
+            #
+            for (my $i = SavedSQLStates() ; $i > 0 ; $i--) {
+                print SQLLOGFID "\t";
+            }
+
             print SQLLOGFID time2str("%D %H:%M:%S $$", time()) . ": $str\n";
         }
         flock(SQLLOGFID,8);     # '8' is magic 'unlock' const.
         close SQLLOGFID;
     }
 }
-    
+
 sub SendSQL {
     my ($str, $dontshadow) = (@_);
     my $iswrite =  ($str =~ /^(INSERT|REPLACE|UPDATE|DELETE)/i);
@@ -130,8 +169,9 @@ sub SendSQL {
         $str =~ s/^LOCK TABLES/LOCK TABLES shadowlog WRITE, /i;
     }
     SqlLog($str);
-    $::currentquery = $::db->query($str)
-	|| die "$str: " . $::db->errmsg;
+    $::currentquery = $::db->prepare($str);
+    $::currentquery->execute
+	|| die "$str: " . $::db->errstr;
     SqlLog("Done");
     if (!$dontshadow && $iswrite && Param("shadowdb")) {
         my $q = SqlQuote($str);
@@ -152,7 +192,7 @@ sub MoreSQLData {
     if (defined @::fetchahead) {
 	return 1;
     }
-    if (@::fetchahead = $::currentquery->fetchrow()) {
+    if (@::fetchahead = $::currentquery->fetchrow_array) {
 	return 1;
     }
     return 0;
@@ -164,7 +204,7 @@ sub FetchSQLData {
 	undef @::fetchahead;
 	return @result;
     }
-    return $::currentquery->fetchrow();
+    return $::currentquery->fetchrow_array;
 }
 
 
@@ -411,10 +451,20 @@ sub GenerateVersionTable {
     @::legal_opsys = SplitEnumType($cols->{"op_sys,type"});
     @::legal_bug_status = SplitEnumType($cols->{"bug_status,type"});
     @::legal_resolution = SplitEnumType($cols->{"resolution,type"});
-    @::legal_resolution_no_dup = @::legal_resolution;
-    my $w = lsearch(\@::legal_resolution_no_dup, "DUPLICATE");
+
+    # 'settable_resolution' is the list of resolutions that may be set 
+    # directly by hand in the bug form. Start with the list of legal 
+    # resolutions and remove 'MOVED' and 'DUPLICATE' because setting 
+    # bugs to those resolutions requires a special process.
+    #
+    @::settable_resolution = @::legal_resolution;
+    my $w = lsearch(\@::settable_resolution, "DUPLICATE");
     if ($w >= 0) {
-        splice(@::legal_resolution_no_dup, $w, 1);
+        splice(@::settable_resolution, $w, 1);
+    }
+    my $z = lsearch(\@::settable_resolution, "MOVED");
+    if ($z >= 0) {
+        splice(@::settable_resolution, $z, 1);
     }
 
     my @list = sort { uc($a) cmp uc($b)} keys(%::versions);
@@ -438,9 +488,10 @@ sub GenerateVersionTable {
     @::legal_components = sort {uc($a) cmp uc($b)} keys(%carray);
     print FID GenerateCode('@::legal_components');
     foreach my $i('product', 'priority', 'severity', 'platform', 'opsys',
-                  'bug_status', 'resolution', 'resolution_no_dup') {
+                  'bug_status', 'resolution') {
         print FID GenerateCode('@::legal_' . $i);
     }
+    print FID GenerateCode('@::settable_resolution');
     print FID GenerateCode('%::proddesc');
     print FID GenerateCode('%::prodmaxvotes');
     print FID GenerateCode('$::anyvotesallowed');
@@ -471,8 +522,9 @@ sub GenerateVersionTable {
     SendSQL("SELECT id, name FROM keyworddefs ORDER BY name");
     while (MoreSQLData()) {
         my ($id, $name) = FetchSQLData();
-        $::keywordsbyname{$name} = $id;
         push(@::legal_keywords, $name);
+        $name = lc($name);
+        $::keywordsbyname{$name} = $id;
     }
     print FID GenerateCode('@::legal_keywords');
     print FID GenerateCode('%::keywordsbyname');
@@ -484,6 +536,14 @@ sub GenerateVersionTable {
 }
 
 
+sub GetKeywordIdFromName {
+    my ($name) = (@_);
+    $name = lc($name);
+    return $::keywordsbyname{$name};
+}
+
+
+
 
 # Returns the modification time of a file.
 
@@ -525,6 +585,9 @@ sub InsertNewUser {
     for (my $i=0 ; $i<8 ; $i++) {
         $password .= substr("abcdefghijklmnopqrstuvwxyz", int(rand(26)), 1);
     }
+    my $usenewemailtech = Param('newemailtech') & Param('newemailtechdefault');
+
+    PushGlobalSQLState();
     SendSQL("select bit, userregexp from groups where userregexp != ''");
     my $groupset = "0";
     while (MoreSQLData()) {
@@ -542,14 +605,17 @@ sub InsertNewUser {
             
     $username = SqlQuote($username);
     $realname = SqlQuote($realname);
-    SendSQL("insert into profiles (login_name, realname, password, cryptpassword, groupset) values ($username, $realname, '$password', encrypt('$password'), $groupset)");
+    SendSQL("insert into profiles (login_name, realname, password, cryptpassword, groupset, newemailtech) values ($username, $realname, '$password', encrypt('$password'), $groupset, $usenewemailtech)");
+    PopGlobalSQLState();
     return $password;
 }
 
 sub DBID_to_real_or_loginname {
     my ($id) = (@_);
+    PushGlobalSQLState();
     SendSQL("SELECT login_name,realname FROM profiles WHERE userid = $id");
     my ($l, $r) = FetchSQLData();
+    PopGlobalSQLState();
     if (!defined $r || $r eq "") {
         return $l;
     } else {
@@ -560,8 +626,10 @@ sub DBID_to_real_or_loginname {
 sub DBID_to_name {
     my ($id) = (@_);
     if (!defined $::cachedNameArray{$id}) {
+        PushGlobalSQLState();
         SendSQL("select login_name from profiles where userid = $id");
         my $r = FetchOneColumn();
+        PopGlobalSQLState();
         if (!defined $r || $r eq "") {
             $r = "__UNKNOWN__";
         }
@@ -572,8 +640,10 @@ sub DBID_to_name {
 
 sub DBname_to_id {
     my ($name) = (@_);
+    PushGlobalSQLState();
     SendSQL("select userid from profiles where login_name = @{[SqlQuote($name)]}");
     my $r = FetchOneColumn();
+    PopGlobalSQLState();
     if (!defined $r || $r eq "") {
         return 0;
     }
@@ -612,7 +682,7 @@ sub DBNameToIdAndCheck {
 sub quoteUrls {
     my ($knownattachments, $text) = (@_);
     return $text unless $text;
-
+    
     my $base = Param('urlbase');
 
     my $protocol = join '|',
@@ -649,17 +719,25 @@ sub quoteUrls {
         $things[$count++] = $item;
     }
     while ($text =~ s/\bbug(\s|%\#)*(\d+)/"##$count##"/ei) {
+        my $item = $&;
+        my $num = $2;
+        $item = GetBugLink($num, $item);
+        $things[$count++] = $item;
+    }
+    while ($text =~ s/\battachment(\s|%\#)*(\d+)/"##$count##"/ei) {
         my $item = $&;
         my $num = $2;
         $item = value_quote($item); # Not really necessary, since we know
                                     # there's no special chars in it.
-        $item = qq{<A HREF="show_bug.cgi?id=$num">$item</A>};
+        $item = qq{<A HREF="showattachment.cgi?attach_id=$num">$item</A>};
         $things[$count++] = $item;
     }
     while ($text =~ s/\*\*\* This bug has been marked as a duplicate of (\d+) \*\*\*/"##$count##"/ei) {
         my $item = $&;
         my $num = $1;
-        $item =~ s@\d+@<A HREF="show_bug.cgi?id=$num">$num</A>@;
+        my $bug_link;
+        $bug_link = GetBugLink($num, $num);
+        $item =~ s@\d+@$bug_link@;
         $things[$count++] = $item;
     }
     while ($text =~ s/Created an attachment \(id=(\d+)\)/"##$count##"/e) {
@@ -685,6 +763,49 @@ sub quoteUrls {
     return $text;
 }
 
+# This is a new subroutine written 12/20/00 for the purpose of processing a
+# link to a bug.  It can be called using "GetBugLink (<BugNumber>, <LinkText>);"
+# Where <BugNumber> is the number of the bug and <LinkText> is what apprears
+# between '<a>' and '</a>'.
+
+sub GetBugLink {
+    my ($bug_num, $link_text) = (@_);
+    my ($link_return) = "";
+
+    # TODO - Add caching capabilites... possibly use a global variable in the form
+    # of $buglink{$bug_num} that contains the text returned by this sub.  If that
+    # variable is defined, simply return it's value rather than running the SQL
+    # query.  This would cut down on the number of SQL calls when the same bug is
+    # referenced multiple times.
+    
+    # Make sure any unfetched data from a currently running query
+    # is saved off rather than overwritten
+    PushGlobalSQLState();
+    
+    # Get this bug's info from the SQL Database
+    SendSQL("select bugs.bug_status, resolution, short_desc, groupset
+             from bugs where bugs.bug_id = $bug_num");
+    my ($bug_stat, $bug_res, $bug_desc, $bug_grp) = (FetchSQLData());
+    
+    # Format the retrieved information into a link
+    if ($bug_stat eq "UNCONFIRMED") { $link_return .= "<i>" }
+    if ($bug_res ne "") { $link_return .= "<strike>" }
+    $bug_desc = value_quote($bug_desc);
+    $link_text = value_quote($link_text);
+    $link_return .= qq{<a href="show_bug.cgi?id=$bug_num" title="$bug_stat};
+    if ($bug_res ne "") {$link_return .= " $bug_res"}
+    if ($bug_grp == 0) { $link_return .= " - $bug_desc" }
+    $link_return .= qq{">$link_text</a>};
+    if ($bug_res ne "") { $link_return .= "</strike>" }
+    if ($bug_stat eq "UNCONFIRMED") { $link_return .= "</i>"}
+    
+    # Put back any query in progress
+    PopGlobalSQLState();
+
+    return $link_return; 
+
+}
+
 sub GetLongDescriptionAsText {
     my ($id, $start, $end) = (@_);
     my $result = "";
@@ -710,7 +831,7 @@ sub GetLongDescriptionAsText {
     while (MoreSQLData()) {
         my ($who, $when, $text) = (FetchSQLData());
         if ($count) {
-            $result .= "\n\n------- Additional Comments From $who  " .
+            $result .= "\n\n------- Additional Comments From $who".Param('emailsuffix')."  ".
                 time2str("%Y-%m-%d %H:%M", str2time($when)) . " -------\n";
         }
         $result .= $text;
@@ -751,6 +872,7 @@ sub GetLongDescriptionAsHTML {
     SendSQL($query);
     while (MoreSQLData()) {
         my ($who, $email, $when, $text) = (FetchSQLData());
+        $email .= Param('emailsuffix');
         if ($count) {
             $result .= "<BR><BR><I>------- Additional Comments From ";
               if ($who) {
@@ -878,7 +1000,11 @@ sub RemoveVotes {
     if (0 < @list) {
         foreach my $ref (@list) {
             my ($name, $count) = (@$ref);
-            if (open(SENDMAIL, "|/usr/lib/sendmail -t")) {
+            my $sendmailparm = '-ODeliveryMode=deferred';
+            if (Param('sendmailnow')) {
+               $sendmailparm = '';
+            }
+            if (open(SENDMAIL, "|/usr/lib/sendmail $sendmailparm -t")) {
                 my %substs;
                 $substs{"to"} = $name;
                 $substs{"bugid"} = $id;
