@@ -1,4 +1,4 @@
-#!/usr/bin/perl -wT
+#!/usr/bonsaitools/bin/perl -w
 # -*- Mode: perl; indent-tabs-mode: nil -*-
 #
 # The contents of this file are subject to the Mozilla Public
@@ -20,22 +20,11 @@
 #
 # Contributor(s): Dawn Endico    <endico@mozilla.org>
 #                 Terry Weissman <terry@mozilla.org>
-#                 Dave Miller    <justdave@bugzilla.org>
 
+use diagnostics;
 use strict;
-
-use lib qw(.);
-
+use Bug;
 require "CGI.pl";
-
-use vars qw($template $userid);
-
-use Bugzilla;
-use Bugzilla::Constants;
-use Bugzilla::Bug;
-use Bugzilla::Config qw(:DEFAULT $datadir);
-use Bugzilla::BugMail;
-
 $::lockcount = 0;
 
 unless ( Param("move-enabled") ) {
@@ -44,14 +33,12 @@ unless ( Param("move-enabled") ) {
   exit;
 }
 
-Bugzilla->login(LOGIN_REQUIRED);
-
-my $cgi = Bugzilla->cgi;
+ConnectToDatabase();
 
 sub Log {
     my ($str) = (@_);
     Lock();
-    open(FID, ">>$datadir/maillog") || die "Can't write to $datadir/maillog";
+    open(FID, ">>data/maillog") || die "Can't write to data/maillog";
     print FID time2str("%D %H:%M", time()) . ": $str\n";
     close FID;
     Unlock();
@@ -60,13 +47,17 @@ sub Log {
 sub Lock {
     if ($::lockcount <= 0) {
         $::lockcount = 0;
-        open(LOCKFID, ">>$datadir/maillock") || die "Can't open $datadir/maillock: $!";
+        if (!open(LOCKFID, ">>data/maillock")) {
+            mkdir "data", 0777;
+            chmod 0777, "data";
+            open(LOCKFID, ">>data/maillock") || die "Can't open lockfile.";
+        }
         my $val = flock(LOCKFID,2);
         if (!$val) { # '2' is magic 'exclusive lock' const.
-            print $cgi->header();
+            print "Content-type: text/html\n\n";
             print "Lock failed: $val\n";
         }
-        chmod 0666, "$datadir/maillock";
+        chmod 0666, "data/maillock";
     }
     $::lockcount++;
 }
@@ -79,8 +70,8 @@ sub Unlock {
     }
 }
 
-if (!defined $cgi->param('buglist')) {
-  print $cgi->header();
+if ( !defined $::FORM{'buglist'} ) {
+  print "Content-type: text/html\n\n";
   PutHeader("Move Bugs");
   print "Move bugs either from the bug display page or perform a ";
   print "<A HREF=\"query.cgi\">query</A> and change several bugs at once.\n";
@@ -90,47 +81,45 @@ if (!defined $cgi->param('buglist')) {
   exit;
 }
 
-my $exporter = Bugzilla->user->login;
+confirm_login();
+my $exporter = $::COOKIE{"Bugzilla_login"};
 my $movers = Param("movers");
-$movers =~ s/\s?,\s?/|/g;
+$movers =~ s/\w?,\w?/|/g;
 $movers =~ s/@/\@/g;
 unless ($exporter =~ /($movers)/) {
-  print $cgi->header();
+  print "Content-type: text/html\n\n";
   PutHeader("Move Bugs");
   print "<P>You do not have permission to move bugs<P>\n";
   PutFooter();
   exit;
 }
 
-my @bugs;
-
+my $xml = "";
+$xml .= Bug::XML_Header( Param("urlbase"), $::param{'version'}, 
+                         Param("maintainer"), $exporter );
 print "<P>\n";
-foreach my $id (split(/:/, scalar($cgi->param('buglist')))) {
-  my $bug = new Bugzilla::Bug($id, $::userid);
-  push @bugs, $bug;
+foreach my $id (split(/:/, $::FORM{'buglist'})) {
+  my $bug = new Bug($id, $::userid);
+  $xml .= $bug->emitXML;
   if (!$bug->error) {
     my $exporterid = DBNameToIdAndCheck($exporter);
 
     my $fieldid = GetFieldID("bug_status");
     my $cur_status= $bug->bug_status;
     SendSQL("INSERT INTO bugs_activity " .
-            "(bug_id,who,bug_when,fieldid,removed,added) VALUES " .
+            "(bug_id,who,bug_when,fieldid,oldvalue,newvalue) VALUES " .
             "($id,$exporterid,now(),$fieldid,'$cur_status','RESOLVED')");
-    $fieldid = GetFieldID("resolution");
+    my $fieldid = GetFieldID("resolution");
     my $cur_res= $bug->resolution;
     SendSQL("INSERT INTO bugs_activity " .
-            "(bug_id,who,bug_when,fieldid,removed,added) VALUES " .
+            "(bug_id,who,bug_when,fieldid,oldvalue,newvalue) VALUES " .
             "($id,$exporterid,now(),$fieldid,'$cur_res','MOVED')");
 
     SendSQL("UPDATE bugs SET bug_status =\"RESOLVED\" where bug_id=\"$id\"");
     SendSQL("UPDATE bugs SET resolution =\"MOVED\" where bug_id=\"$id\"");
 
-    my $comment = "";
-    if (defined $cgi->param('comment') && $cgi->param('comment') !~ /^\s*$/) {
-        $comment .= $cgi->param('comment') . "\n\n";
-    }
-    $comment .= "Bug moved to " . Param("move-to-url") . ".\n\n";
-    $comment .= "If the move succeeded, $exporter will receive a mail\n";
+    my $comment = "Bug moved to " . Param("move-to-url") . ".\n\n";
+    $comment .= "If the move succeeded, $exporter will recieve a mail\n";
     $comment .= "containing the number of the new bug in the other database.\n";
     $comment .= "If all went well,  please mark this bug verified, and paste\n";
     $comment .= "in a link to the new bug. Otherwise, reopen this bug.\n";
@@ -138,12 +127,13 @@ foreach my $id (split(/:/, scalar($cgi->param('buglist')))) {
         "($id,  $exporterid, now(), " . SqlQuote($comment) . ")");
 
     print "<P>Bug $id moved to " . Param("move-to-url") . ".<BR>\n";
-    Bugzilla::BugMail::Send($id, { 'changer' => $exporter });
+    system("./processmail", $id, $exporter);
   }
 }
 print "<P>\n";
+$xml .= Bug::XML_Footer;
 
-my $buglist = $cgi->param('buglist');
+my $buglist = $::FORM{'buglist'};
 $buglist =~ s/:/,/g;
 my $host = Param("urlbase");
 $host =~ s#http://([^/]+)/.*#$1#;
@@ -154,21 +144,13 @@ my $from = Param("moved-from-address");
 $from =~ s/@/\@/;
 $msg .= "From: Bugzilla <" . $from . ">\n";
 $msg .= "Subject: Moving bug(s) $buglist\n\n";
+$msg .= $xml . "\n";
 
-my @fieldlist = (Bugzilla::Bug::fields(), 'group', 'long_desc', 'attachment');
-my %displayfields;
-foreach (@fieldlist) {
-    $displayfields{$_} = 1;
-}
-
-$template->process("bug/show.xml.tmpl", { bugs => \@bugs,
-                                          displayfields => \%displayfields,
-                                        }, \$msg)
-  || ThrowTemplateError($template->error());
-
-$msg .= "\n";
-
-Bugzilla::BugMail::MessageToMTA($msg);
+open(SENDMAIL,
+  "|/usr/lib/sendmail -ODeliveryMode=background -t") ||
+    die "Can't open sendmail";
+print SENDMAIL $msg;
+close SENDMAIL;
 
 my $logstr = "XML: bugs $buglist sent to $to";
 Log($logstr);
